/*
 * VEML6046X00_Application_Library.cpp
 *
 * Created  : 6  March 2023
 * Modified : 14 June 2023
 * Author   : HWanyusof
 * Version	: 1.0
 */

#include "VEML6046X00_Prototypes.h"
#include "VEML6046X00.h"
#include "I2C_Functions.h"
#include "VEML6046X00_Application_Library.h"

extern int I2C_Bus;

//****************************************************************************************************
//***************************************Application API**********************************************


/*Determine Resolution (in lux/count) From RGB_IT, RGB_GAIN and PD_DIV
 *VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN)
 *Byte PD_DIV - Input Parameter:
 *
 * VEML6046X00_PD_DIV_FULL
 * VEML6046X00_PD_DIV_HALF
 *
 *Byte RGB_IT - Input Parameter:
 *
 * VEML6046X00_RGB_IT_3_125ms
 * VEML6046X00_RGB_IT_6_25ms
 * VEML6046X00_RGB_IT_12_5ms
 * VEML6046X00_RGB_IT_25ms
 * VEML6046X00_RGB_IT_50ms
 * VEML6046X00_RGB_IT_100ms
 * VEML6046X00_RGB_IT_200ms
 * VEML6046X00_RGB_IT_400ms
 *
 *Byte RGB_GAIN - Input Parameter:
 *
 * VEML6046X00_RGB_GAIN_x0_5
 * VEML6046X00_RGB_GAIN_x0_66
 * VEML6046X00_RGB_GAIN_x1
 * VEML6046X00_RGB_GAIN_x2
 *
 *Info: Refer to Table 12 & 13 in Datasheet Page 11
 *
 *returns resolution (lux/count)
 */
float VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN)
{
    float Resolution;
    // For the whole PD size (High sensitivity)
    if(PD_DIV == VEML6046X00_PD_DIV_FULL)
    {
        if (RGB_IT == VEML6046X00_RGB_IT_400ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0053;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0105;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.0159;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.0210;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_200ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0105;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0210;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.0318;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.0420;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_100ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0210;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0420;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.0636;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.0840;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_50ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0420;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0840;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.1273;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.1680;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_25ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0840;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.1680;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.2545;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.3360;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_12_5ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.1680;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.3360;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.5091;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.6720;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_6_25ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.3360;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.6720;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 1.0182;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 1.3440;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_3_125ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.6720;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 1.3440;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 2.0364;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 2.6880;
        }
    }
    // For the 1/2 PD size (Half sensitivity)
    if(PD_DIV == VEML6046X00_PD_DIV_HALF)
    {
        if (RGB_IT == VEML6046X00_RGB_IT_400ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0105;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0210;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.0318;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.0420;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_200ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0210;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0420;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.0636;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.0840;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_100ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0420;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.0840;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.1273;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.1680;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_50ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.0840;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.1680;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.2545;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.3360;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_25ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.1680;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.3360;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 0.5091;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 0.6720;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_12_5ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.3360;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 0.6720;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 1.0182;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 1.3440;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_6_25ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 0.6720;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 1.3440;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 2.0364;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 2.6880;
        }
        if (RGB_IT == VEML6046X00_RGB_IT_3_125ms)
        {
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x2) Resolution = 1.3440;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x1) Resolution = 2.6880;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66) Resolution = 4.0727;
            if (RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5) Resolution = 5.3760;
        }
    }
    return(Resolution);
}

/*Calculate the lux of the green channel
 *VEML6046X00_CAL_Lux(float Resolution, float Count)
 *float Resolution - output from VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN)
 *float Count - output from VEML6046X00_GET_G_DATA()
 *
 *returns lux
 */
float VEML6046X00_CAL_Lux(float Resolution, float Count)
{
    float Lux;
    Lux = Resolution*Count;
    return(Lux);
}

/*Get Delay for Measurement
 *VEML6046X00_GET_Delay()
 *returns delay in ms
 */
int VEML6046X00_GET_Delay()
{
	int Delay;

	//Delay for RGB
	//Delay = Delay from IT + Circuit (~10ms)
	if (VEML6046X00_GET_RGB_IT_Bits() == 1) {Delay = 15;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 2) {Delay = 20;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 3) {Delay = 25;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 4) {Delay = 40;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 5) {Delay = 80;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 6) {Delay = 150;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 7) {Delay = 280;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 8) {Delay = 500;}

	return Delay;
}

//Wait for measurement function
void Wait()
{
   delay(VEML6046X00_GET_Delay()); 
}

/*Get RGB_IT
 *VEML6046X00_GET_RGB_IT()
 *returns RGB_IT
 */
Byte VEML6046X00_GET_RGB_IT()
{

	if (VEML6046X00_GET_RGB_IT_Bits() == 1) {return VEML6046X00_RGB_IT_3_125ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 2) {return VEML6046X00_RGB_IT_6_25ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 3) {return VEML6046X00_RGB_IT_12_5ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 4) {return VEML6046X00_RGB_IT_25ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 5) {return VEML6046X00_RGB_IT_50ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 6) {return VEML6046X00_RGB_IT_100ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 7) {return VEML6046X00_RGB_IT_200ms;}
	if (VEML6046X00_GET_RGB_IT_Bits() == 8) {return VEML6046X00_RGB_IT_400ms;}
	else
	return 0;
}

/*Get RGB_GAIN
 *VEML6046X00_GET_RGB_GAIN()
 *returns RGB_GAIN
 */
Byte VEML6046X00_GET_RGB_GAIN()
{

	if (VEML6046X00_GET_RGB_GAIN_Bits() == 1) {return VEML6046X00_RGB_GAIN_x1;}
	if (VEML6046X00_GET_RGB_GAIN_Bits() == 2) {return VEML6046X00_RGB_GAIN_x2;}
	if (VEML6046X00_GET_RGB_GAIN_Bits() == 3) {return VEML6046X00_RGB_GAIN_x0_66;}
	if (VEML6046X00_GET_RGB_GAIN_Bits() == 4) {return VEML6046X00_RGB_GAIN_x0_5;}
	else
	return 0;
}

/*Get PD_DIV
 *VEML6046X00_GET_PD_DIV()
 *returns PD_DIV
 */
Byte VEML6046X00_GET_PD_DIV()
{

	if (VEML6046X00_GET_PD_DIV_Bit() == 0) {return VEML6046X00_PD_DIV_FULL;}
	if (VEML6046X00_GET_PD_DIV_Bit() == 1) {return VEML6046X00_PD_DIV_HALF;}
	else
	return 0;
}

/*Get RGB Mode
 *returns the RGB mode status of the sensor as follows:
 *
 * 0 - RGB Shutdown Mode
 * 1 - AF Mode
 * 2 - Auto Mode
 *
 */
int VEML6046X00_GET_RGB_Mode()
{
	int Mode;

	//Read the RGB_ON bits: Mode = 0 - RGB Shutdown
	if ((VEML6046X00_GET_RGB_ON_0_Bit() == 0b1) && (VEML6046X00_GET_RGB_ON_1_Bit() == 0b1)) Mode = 0;

	//Read the RGB_MODE bit: Mode = 1 - AF Mode
	if ((VEML6046X00_GET_RGB_ON_0_Bit() == 0b0) && (VEML6046X00_GET_RGB_ON_1_Bit() == 0b0) && (VEML6046X00_GET_RGB_MODE_Bit() == 0b1)) Mode = 1;

	//Read the RGB_MODE bit: Mode = 2 - Auto Mode
	if ((VEML6046X00_GET_RGB_ON_0_Bit() == 0b0) && (VEML6046X00_GET_RGB_ON_1_Bit() == 0b0) && (VEML6046X00_GET_RGB_MODE_Bit() == 0b0)) Mode = 2;

	return Mode;
}

//Reset the Sensor to the default value
void Reset_Sensor()
{
	struct TransferData VEML6046X00_Data;
	VEML6046X00_Data.Slave_Address = VEML6046X00_Slave_Address;
	VEML6046X00_Data.RegisterAddress = VEML6046X00_RGB_CONF_0;
	VEML6046X00_Data.Select_I2C_Bus = I2C_Bus;
	VEML6046X00_Data.WData[0] = 0x01;
	VEML6046X00_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VEML6046X00_Data);

	VEML6046X00_Data.Slave_Address = VEML6046X00_Slave_Address;
	VEML6046X00_Data.RegisterAddress = VEML6046X00_RGB_THDH_L;
	VEML6046X00_Data.Select_I2C_Bus = I2C_Bus;
	VEML6046X00_Data.WData[0] = 0x00;
	VEML6046X00_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VEML6046X00_Data);

	VEML6046X00_Data.Slave_Address = VEML6046X00_Slave_Address;
	VEML6046X00_Data.RegisterAddress = VEML6046X00_RGB_THDL_L;
	VEML6046X00_Data.Select_I2C_Bus = I2C_Bus;
	VEML6046X00_Data.WData[0] = 0x00;
	VEML6046X00_Data.WData[1] = 0x00;
	WriteI2C_Bus(&VEML6046X00_Data);
}

//Print the output of the sensor
void Print_Data_Only()
{
	/* Print the sensor output */
	int Error = 0;
	Word R_Data;
	Word G_Data;
	Word B_Data;
	Word IR_Data;
	float Resolution;
	float Lux;
	Byte RGB_IT;
	Byte RGB_GAIN;
	Byte PD_DIV;
	Byte Interrupt;

	delay(300);

	//Active Force Mode
	if(VEML6046X00_GET_RGB_Mode() == 1)
	{
		//Set the active force mode trigger
		VEML6046X00_SET_RGB_TRIG(VEML6046X00_RGB_TRIG_EN);

		//Wait for data ready flag
		while(!VEML6046X00_GET_AF_DATA_READY_FLAG())
		{
			Wait();
		}
	}

	//Auto Mode
	if(VEML6046X00_GET_RGB_Mode() == 2)
	{
		//Delay of IT ms + other Circuit Delay (~10ms)
		Wait();
	}

	//Read the RGB channels output data
	R_Data = VEML6046X00_GET_R_DATA();
	G_Data = VEML6046X00_GET_G_DATA();
	B_Data = VEML6046X00_GET_B_DATA();

	//Read the needed parameters to determine the resolution for green channel
	RGB_IT = VEML6046X00_GET_RGB_IT();
	RGB_GAIN = VEML6046X00_GET_RGB_GAIN();
	PD_DIV = VEML6046X00_GET_PD_DIV();

	//Determine Resolution (in lux/count) From RGB_IT, RGB_GAIN and PD_DIV
	Resolution = VEML6046X00_CAL_Resolution(PD_DIV, RGB_IT, RGB_GAIN);

	//Calculate Lux for the green channel
	Lux = VEML6046X00_CAL_Lux(Resolution, G_Data);

	//Read the IR channel
	IR_Data = VEML6046X00_GET_IR_DATA();

	/* Print the red channel data */
	Serial.print("Red Channel : ");
	Serial.print(R_Data,DEC);
	Serial.println(" Counts ");
	delay(50);

	/* Print the green channel data */
	Serial.print("Green Channel : ");
	Serial.print(G_Data,DEC);
	Serial.println(" Counts ");
	delay(50);

	/* Print the blue channel data */
	Serial.print("Blue Channel : ");
	Serial.print(B_Data,DEC);
	Serial.println(" Counts ");
	delay(50);

	/* Print the resolution for the green channel */
	Serial.print("Resolution (Green Channel) : ");
	Serial.print(Resolution,5);
	Serial.println(" Lux/Count ");
	delay(50);

	/* Print the lux values */
	Serial.print("Lux (Green Channel) : ");
	Serial.print(Lux,5);
	Serial.println(" Lux ");
	delay(50);

	/* Print the IR channel data */
	Serial.print("IR Channel : ");
	Serial.print(IR_Data,DEC);
	Serial.println(" Counts ");
	delay(50);

	//Auto Mode
	if(VEML6046X00_GET_RGB_Mode() == 2)
	{
		//Read interrupt flag register
		Interrupt = VEML6046X00_GET_INT_FLAG();
		delay(50);

		/* Print the interrupt register */
		Serial.print("Interrupt : 0b");
		Serial.println(Interrupt,BIN);
		delay(50);
		
	}

	Serial.println("");
	delay(50);

	if (VEML6046X00_GET_ID() != 0x01)
	{
		Error = 1;
		R_Data = 0;
		G_Data = 0;
		B_Data = 0;
		IR_Data = 0;
		Lux = 0.0;
		Resolution = 0.0;
	}
	else if (Error == 1)
	{
		//Reinitialize after error
		//Can be changed to Auto_Mode() or AF_Mode if being used in INIT_VEML6046X00() above
		Basic_Initialization_Auto_Mode();

		delay(50);

		//Reset error
		Error = 0;
	}
}

//Execute automatic gain control for the green channel
Word VEML6046X00_AGC()
{
    Word G_Data_AGC;
    Byte RGB_IT;
    Byte RGB_GAIN;
    Byte PD_DIV;

    //Check ALS count and change the integration time, gain and photodiode size to optimize the counts to be within upper and lower limit
    for(;;)
    {
        //Active Force Mode
        if(VEML6046X00_GET_RGB_Mode() == 1)
    	{
    		//Set Trigger for AF Mode
    		VEML6046X00_SET_RGB_TRIG(VEML6046X00_RGB_TRIG_EN);
    	}

        //Wait for the measurement to be completed
        Wait();

    	//Read the green channel data
        G_Data_AGC = VEML6046X00_GET_G_DATA();

        //Read the needed parameters to determine the resolution for green channel
        RGB_IT = VEML6046X00_GET_RGB_IT();
        RGB_GAIN = VEML6046X00_GET_RGB_GAIN();
        PD_DIV = VEML6046X00_GET_PD_DIV();

        //If the green channel data is lower than the lower limit -> increase the integration time, gain and photodiode size until the counts and resolution are optimized
        if(G_Data_AGC < 25000)
        {
        	if(RGB_IT == VEML6046X00_RGB_IT_3_125ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_6_25ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_6_25ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_12_5ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_12_5ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_25ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_25ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_50ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_50ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_100ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_100ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_200ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_200ms)
			{
				VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_400ms);
				continue;
			}

        	else if(RGB_IT == VEML6046X00_RGB_IT_400ms)
			{
				if(RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x0_66);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x1);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x1)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x2);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x2)
				{
					if(PD_DIV == VEML6046X00_PD_DIV_HALF)
					{
						VEML6046X00_SET_PD_DIV(VEML6046X00_PD_DIV_FULL);
						continue;
					}
					else if(PD_DIV == VEML6046X00_PD_DIV_FULL)
					{
                        Wait();
						break;
					}
				}
			}
        }

        //If the green channel data is within the lower and upper limit -> break the loop since the counts and resolution are optimized
        else if((G_Data_AGC >= 25000)&&(G_Data_AGC <= 55000))
        {
        	Wait();
            break;
        }
        //If the green channel data is higher than the upper limit -> reduce the integration time, gain and photodiode size until the counts and resolution are optimized
        else if(G_Data_AGC > 55000)
        {

            if(RGB_IT == VEML6046X00_RGB_IT_400ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_200ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_200ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_100ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_100ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_50ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_50ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_25ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_25ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_12_5ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_12_5ms)
            {
                VEML6046X00_SET_RGB_IT(VEML6046X00_RGB_IT_6_25ms);
                continue;
            }

            else if(RGB_IT == VEML6046X00_RGB_IT_6_25ms)
            {
            	if(RGB_GAIN == VEML6046X00_RGB_GAIN_x2)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x1);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x1)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x0_66);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x0_66)
				{
					VEML6046X00_SET_RGB_GAIN(VEML6046X00_RGB_GAIN_x0_5);
					continue;
				}

				else if(RGB_GAIN == VEML6046X00_RGB_GAIN_x0_5)
				{
					if(PD_DIV == VEML6046X00_PD_DIV_FULL)
					{
						VEML6046X00_SET_PD_DIV(VEML6046X00_PD_DIV_HALF);
						continue;
					}
					else if(PD_DIV == VEML6046X00_PD_DIV_HALF)
					{
						Wait();
						break;
					}
				 }
            }
        }
    }

    return G_Data_AGC;
}

//Print the output of the sensor using AGC
void Print_Data_Only_AGC()
{
	int Error = 0;
	Word G_Data;
	float Resolution;
	float Lux;
	Byte RGB_IT;
	Byte RGB_GAIN;
	Byte PD_DIV;

	//Clear Interrupt Flag
	VEML6046X00_GET_INT_FLAG();

	if(VEML6046X00_GET_RGB_Mode() == 1)
	{
		//Set Trigger for AF Mode
		VEML6046X00_SET_RGB_TRIG(VEML6046X00_RGB_TRIG_EN);
	}

	//Delay of IT ms + other Circuit Delay (~10ms)
	Wait();

	//Read the green channel output data
	G_Data = VEML6046X00_AGC();

	//Read the needed parameters to determine the resolution for green channel
	RGB_IT = VEML6046X00_GET_RGB_IT();
	RGB_GAIN = VEML6046X00_GET_RGB_GAIN();
	PD_DIV = VEML6046X00_GET_PD_DIV();

	//Determine Resolution (in lux/count) From RGB_IT, RGB_GAIN and PD_DIV
	Resolution = VEML6046X00_CAL_Resolution(PD_DIV, RGB_IT, RGB_GAIN);

	//Calculate Lux for the green channel
	Lux = VEML6046X00_CAL_Lux(Resolution, G_Data);

	/* Print the green channel data */
	Serial.print("Green Channel : ");
	Serial.print(G_Data,DEC);
	Serial.println(" Counts ");
	delay(50);

	/* Print the resolution for the green channel */
	Serial.print("Resolution (Green Channel) : ");
	Serial.print(Resolution,5);
	Serial.println(" Lux/Count ");
	delay(50);

	/* Print the lux values */
	Serial.print("Lux (Green Channel) : ");
	Serial.print(Lux,5);
	Serial.println(" Lux ");
	delay(50);

	Serial.println("");
	delay(5);

	if (VEML6046X00_GET_ID() != 0x01)
	{
		Error = 1;
		G_Data = 0;
		Lux = 0.0;
		Resolution = 0.0;
	}
	else if (Error == 1)
	{
		//Reinitialize after error
		//Can be changed to Auto_Mode() or AF_Mode if being used in INIT_VEML6046X00() above
		Basic_Initialization_Auto_Mode();

		delay(50);

		//Reset error
		Error = 0;
	}
}

/*Print the variable in DEC for debugging
 *Print_Variable_DEC(Word Var)
 *Word Var - Input Parameter:
 *
 * Value between 0d0 and 0d65535
 */
void Print_Variable_DEC(Word Var)
{
	Serial.println("***************************************************");
	delay(50);

	Serial.print(">>>>>>>Variable : 0d");
	Serial.println(Var,DEC);
	delay(50);

	Serial.println("***************************************************");

	delay(2000);
}

/*Print the variable in HEX for debugging
 *Print_Variable_HEX(Word Var)
 *Word Var - Input Parameter:
 *
 * Value between 0d0 and 0d65535
 */
void Print_Variable_HEX(Word Var)
{
	Serial.println("***************************************************");
	delay(50);

	Serial.print(">>>>>>>Variable : 0x");
	Serial.println(Var,HEX);
	delay(50);

	Serial.println("***************************************************");

	delay(2000);
}

/*Reverses a string 'str' of length 'len'
 *reverse(char* str, int len)
 *char* str - Array pointer to be reversed
 *int len - Length of the array
 */
void reverse(char* str, int len)
{
    int i = 0, j = len - 1, temp;
    while (i < j) {
        temp = str[i];
        str[i] = str[j];
        str[j] = temp;
        i++;
        j--;
    }
}

/*Converts a given integer x to string str[].
 *intToStr(int x, char str[], int d)
 *int x - floating-point number to be converted to a string (Both the integer part as well as fraction/decimal point part)
 *char str[] - output string of the floating-point number in the form of array character
 *int d - number of decimal point (the integer part always = 0, the fraction/decimal point part = int afterpoint from ftoa() input)
 */
int intToStr(int x, char str[], int d)
{
    int i = 0;

    while (x)
	{
		//Store and convert int to char (Valid for single digit)
        str[i++] = (x % 10) + '0';
        x = x / 10;
    }

    //If number of digits required is more, then
    //add 0s at the beginning
    while (i < d) str[i++] = '0';

	//Reverse the string characters in the array str
    reverse(str, i);

	//Place the null character at the end of the array
    str[i] = '\0';

	//Return the position i
    return i;
}

/*Converts a floating-point/double number to string.
 *ftoa(float n, char* res, int afterpoint)
 *float n - floating-point number to be converted to a string
 *char* res - pointer to output string of the floating-point number in the form of array character
 *int d - number of decimal point
 */
void ftoa(float n, char* res, int afterpoint)
{
    //Extract integer part
    int ipart = (int)n;

    //Extract decimal part
    float fpart = n - (float)ipart;

    //Convert integer part to string and the function returns the position after the interger
    int i = intToStr(ipart, res, 0);

    //Check for display option after point
    if (afterpoint != 0)
	{
		//Add dot after the integer part
        res[i] = '.';

        //Multiply decimal part by 10^decimal point
        fpart = fpart* pow(10, afterpoint);

		//Convert decimal part to string
        intToStr((int)fpart, res + i + 1, afterpoint);
    }
}
