/*
 * VEML6046X00_Application_Library.h
 *
 * Created  : 6 March 2023
 * Modified : 14 June 2023
 * Author   : HWanyusof
 * Version	: 1.0
 */

#include "typedefinition.h"

/*Determine Resolution (in lux/count) From RGB_IT, RGB_GAIN and PD_DIV
 *VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN)
 *Byte PD_DIV - Input Parameter:
 *
 * VEML6046X00_PD_DIV_FULL
 * VEML6046X00_PD_DIV_HALF
 *
 *Byte RGB_IT - Input Parameter:
 *
 * VEML6046X00_RGB_IT_3_125ms
 * VEML6046X00_RGB_IT_6_25ms
 * VEML6046X00_RGB_IT_12_5ms
 * VEML6046X00_RGB_IT_25ms
 * VEML6046X00_RGB_IT_50ms
 * VEML6046X00_RGB_IT_100ms
 * VEML6046X00_RGB_IT_200ms
 * VEML6046X00_RGB_IT_400ms
 *
 *Byte RGB_GAIN - Input Parameter:
 *
 * VEML6046X00_RGB_GAIN_x0_5
 * VEML6046X00_RGB_GAIN_x0_66
 * VEML6046X00_RGB_GAIN_x1
 * VEML6046X00_RGB_GAIN_x2
 *
 *Info: Refer to Table 12 & 13 in Datasheet Page 11
 *
 *returns resolution (lux/count)
 */
float VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN);

/*Calculate the lux of the green channel
 *VEML6046X00_CAL_Lux(float Resolution, float Count)
 *float Resolution - output from VEML6046X00_CAL_Resolution(Byte PD_DIV, Byte RGB_IT, Byte RGB_GAIN)
 *float Count - output from VEML6046X00_GET_G_DATA()
 *
 *returns lux
 */
float VEML6046X00_CAL_Lux(float Resolution, float Count);

/*Get Delay for Measurement
 *VEML6046X00_GET_Delay()
 *returns delay in ms
 */
int VEML6046X00_GET_Delay();

//Wait for measurement function
void Wait();

/*Get RGB_IT
 *VEML6046X00_GET_RGB_IT()
 *returns RGB_IT
 */
Byte VEML6046X00_GET_RGB_IT();

/*Get RGB_GAIN
 *VEML6046X00_GET_RGB_GAIN()
 *returns RGB_GAIN
 */
Byte VEML6046X00_GET_RGB_GAIN();

/*Get PD_DIV
 *VEML6046X00_GET_PD_DIV()
 *returns PD_DIV
 */
Byte VEML6046X00_GET_PD_DIV();

/*Get RGB Mode
 *returns the RGB mode status of the sensor as follows:
 *
 * 0 - RGB Shutdown Mode
 * 1 - AF Mode
 * 2 - Auto Mode
 *
 */
int VEML6046X00_GET_RGB_Mode();

//Reset the Sensor to the default value
void Reset_Sensor();

//Print the output of the sensor
void Print_Data_Only();

//Execute automatic gain control for the green channel
Word VEML6046X00_AGC();

//Print the output of the sensor using AGC
void Print_Data_Only_AGC();

/*Print the variable in DEC for debugging
 *Print_Variable_DEC(Word Var)
 *Word Var - Input Parameter:
 *
 * Value between 0d0 and 0d65535
 */
void Print_Variable_DEC(Word Var);

/*Print the variable in HEX for debugging
 *Print_Variable_HEX(Word Var)
 *Word Var - Input Parameter:
 *
 * Value between 0d0 and 0d65535
 */
void Print_Variable_HEX(Word Var);

/*Reverses a string 'str' of length 'len'
 *reverse(char* str, int len)
 *char* str - Array pointer to be reversed
 *int len - Length of the array
 */
void reverse(char* str, int len);

/*Converts a given integer x to string str[].
 *intToStr(int x, char str[], int d)
 *int x - floating-point number to be converted to a string (Both the integer part as well as fraction/decimal point part)
 *char str[] - output string of the floating-point number in the form of array character
 *int d - number of decimal point (the integer part always = 0, the fraction/decimal point part = int afterpoint from ftoa() input)
 */
int intToStr(int x, char str[], int d);

/*Converts a floating-point/double number to string.
 *ftoa(float n, char* res, int afterpoint)
 *float n - floating-point number to be converted to a string
 *char* res - pointer to output string of the floating-point number in the form of array character
 *int d - number of decimal point
 */
void ftoa(float n, char* res, int afterpoint);

//Auto/Self-Timed Mode Basic Initialization Function
void Basic_Initialization_Auto_Mode();
